// DSPIC.cpp
// CdsPIC is a class created for the 4MHz PIC16LF628-04/SO code 
//   that was written to provide an RS232 to 2-wire or Bit I/O Interface
// The RS232 Communications Routines used in this file are located 
//    in the DSIOLIB1.cpp/.h


//Jason Rauch
//March 25, 2002


//***************************************************************************
//---------------------------------------------------------------------------
// Copyright (C) 2002 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 

#include "stdafx.h"
#include "DSPIC.h"

#define MAX_PORTNUM 1

//externally defined global vars
extern CString m_strComPort;
extern HANDLE ComID[MAX_PORTNUM];
extern OVERLAPPED osRead[MAX_PORTNUM],osWrite[MAX_PORTNUM];

//externally defined global functions
extern bool  AcquireCOMPort(int,char *,char *);
extern void ReleaseCOMPort(int,char *);
extern void FlushCOM(int);
extern int  WriteCOM(int, int, UCHAR *);
extern int  ReadCOM(int, int, UCHAR *);

bool CdsPic::BoardPresent()
{//Call to Determine If Board Was Detected On Power-up
	
	return m_bBoardPresent;
}

CdsPic::CdsPic()
{	// class constructor, opens serial port detects board.
	//  looks for board on COM1, 2, 3, and 4

	// Open COM Port and Detect Board
	m_bComIsOpen = FALSE;
	m_bBoardPresent = FALSE;
	char ComStr[5] = "COM1";
	char ComStrActive[30] = "COM2";
		if(!m_bBoardPresent && DetectBoard(ComStr))
		{
			strcpy(ComStrActive, ComStr);
		}
		else
		{
			if(ComStr[3] == '4')
				ComStr[3] = '1';
			else ComStr[3] = ComStr[3] + 1;
		}
		if(!m_bBoardPresent && DetectBoard(ComStr))
		{
	     	strcpy(ComStrActive, ComStr);
		}
		else
		{
			if(ComStr[3] == '4')
				ComStr[3] = '1';
			else ComStr[3] = ComStr[3] + 1;
		}
		if(!m_bBoardPresent && DetectBoard(ComStr))
		{
	     	strcpy(ComStrActive, ComStr);
		}
		else
		{
			if(ComStr[3] == '4')
				ComStr[3] = '1';
			else ComStr[3] = ComStr[3] + 1;
		}
		if(!m_bBoardPresent && DetectBoard(ComStr))
		{
	        strcpy(ComStrActive, ComStr);
		}
		else
		{
			if(ComStr[3] == '4')
				ComStr[3] = '1';
			else ComStr[3] = ComStr[3] + 1;
		}

		if(!m_bBoardPresent)
		{
			m_bBoardPresent = FALSE;
			m_bComIsOpen = FALSE;
//			AfxMessageBox("The DS2W board could not be found.  Fix and restart application.");
		}
		else
		{
			m_bBoardPresent = TRUE;
			m_bComIsOpen = TRUE;
			strcat(ComStrActive, " Has Been Initialized");	
//			AfxMessageBox(ComStrActive);
		}
}

CdsPic::~CdsPic()
{  // class destructor, closes serial port, resets "present" variables.
	char returnstring[80];
	if(m_bComIsOpen)
	{
		ReleaseCOMPort(0, returnstring);
		//MessageBox(returnstring);

	}
	m_bComIsOpen = FALSE;
	m_bBoardPresent = FALSE;
}

bool CdsPic::DetectBoard(char *port)
{
	// Open COM Port Here
	char returnstring[80];
	int NoBytes;

	m_bComIsOpen = AcquireCOMPort(0, port, returnstring);
	if(!m_bComIsOpen)
	{
		m_bComIsOpen = FALSE;
		m_bBoardPresent = FALSE;
		return FALSE;
	}
	else
	{
		// Port is open
		// Now look for board
		m_bBoardPresent = FALSE;
		Sleep(50);  // was 500  
		FlushCOM(0);
		// Now detect board
		// Now toggle DTR
		if(!ResetBoard())
		{
			// Close port
			ReleaseCOMPort(0, returnstring);
			m_bComIsOpen = FALSE;
			m_bBoardPresent = FALSE;
			return FALSE;
		}

		// DTR toggled fine
		// Read banner
		NoBytes = ReadCOM(0,19,m_strBoardBanner);
		if(NoBytes == 0)
		{
			m_strBoardBanner[0] = 0;
			// Close port
			ReleaseCOMPort(0, returnstring);
			m_bComIsOpen = FALSE;
			m_bBoardPresent = FALSE;
			return FALSE;
		}
		else
		{
			m_strBoardBanner[NoBytes] = 0;
			m_bBoardPresent = TRUE;
			return TRUE;		
		}
	}
	return FALSE;
}

UCHAR * CdsPic::HardwareRevision()
{
	return &m_strBoardBanner[0];
}

bool CdsPic::Read1(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE1;			// Load buf with read1 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read2(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE2;			// Load buf with read2 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read3(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE3;			// Load buf with read3 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read4(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE4;			// Load buf with read4 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read5(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE5;			// Load buf with read5 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read6(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE6;			// Load buf with read6 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read7(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE7;			// Load buf with read7 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read8(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE8;			// Load buf with read8 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}
bool CdsPic::Read9(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xE9;			// Load buf with read9 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read10(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xEA;			// Load buf with read10 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::Read11(bool &state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xEB;			// Load buf with read11 Command
	buf[1]=0x00;			// Second byte dummy byte, required
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] == 0xFA)
		return false;						// return false
	else
	{
		if(buf[0])
			state = true;
		else
			state = false;
		return true;							// else return true
	}
}

bool CdsPic::ReadSlave2W(UCHAR &data, bool ACK)
{	// reads slave w/ or wo/ACK, returns data and success
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]= 0xA2;			// Load buf with 2-wire ReadSlave Command
	buf[1]= ACK;			// Second byte determines ACK/NACK for this command
							// ACK=1, send read with ACK, ACK=0, read with NACK
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,2,buf);		// Read for PIC Command Ack (0xb2h, for readslave)
	if(buf[1] != 0xB2)
	{
		data = buf[0];
		return false;						// and return false
	}
	else
	{
		data = buf[0];
		return true;							// else return true
	}
}

bool CdsPic::ResetBoard()
{
	// Just pulse DTR to reset micro
		if (EscapeCommFunction(ComID[0], CLRDTR))
		{
			if (EscapeCommFunction(ComID[0], SETDTR))
				return TRUE;
			else
				return FALSE;
		}		
		else
			return FALSE;
}

bool CdsPic::Start2W()
{	//Sends Start Command, returns success
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xA0;			// Load buf with 2-wire Start Command
	buf[1]=0x00;			// Second byte dummy byte for this command
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xb0h, for start)
	if(buf[0] != 0xB0)
		return false;						// and return false
	return true;							// else return true
}
bool CdsPic::Stop2W()
{	//Sends Stop Command, returns success
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xA3;			// Load buf with 2-wire Stop Command
	buf[1]=0x00;			// Second byte dummy byte for this command
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xb3h, for stop)
	if(buf[0] != 0xB3)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::ToggleSCL9x()
{	//Sends Toggle SCL9 Command, returns success
	// toggles SCL 9 times to enable the slave to output data and ready for a stop
	// command.  When followed by a stop command the bus should be free and reset.

	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xA4;			// Load buf with 2-wire Toggle SCL 9x Command
	buf[1]=0x00;			// Second byte dummy byte for this command
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xb4h, for toggle9x command)
	if(buf[0] != 0xB4)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write1(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF1;			// Load buf with write1 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write2(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF2;			// Load buf with write2 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write3(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF3;			// Load buf with write3 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write4(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF4;			// Load buf with write4 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write5(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF5;			// Load buf with write5 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write6(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF6;			// Load buf with write6 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write7(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF7;			// Load buf with write7 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write8(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF8;			// Load buf with write8 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write9(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xF9;			// Load buf with write9 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}

bool CdsPic::Write10(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xFA;			// Load buf with write10 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true
}


bool CdsPic::Write11(bool state)
{
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xFB;			// Load buf with write11 Command
	buf[1]=state;			// Second byte pin state to write
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xf0h)
	if(buf[0] != 0xF0)
		return false;						// and return false
	return true;							// else return true

}

bool CdsPic::WriteSlave2W(UCHAR data)
{	// writes data to slave returns success based on slave acknoledgement
	unsigned char buf[2];	// Local Rx/Tx com buffer 
	FlushCOM(0);			// Clear COM Port Hardware Buffers
	buf[0]=0xA1;			// Load buf with 2-wire Write Slave Command
	buf[1]=data;			// Second byte data byte for this command
	WriteCOM(0,2,buf);		// Send Data via Serial Port
	ReadCOM(0,1,buf);		// Read for PIC Command Ack (0xb1h, for write slave command)
	if(buf[0] != 0xB1)
		return false;						// and return false
	return true;							// else return true
}